package FindNewSeries;
use 5.014;
use Moose;
use namespace::autoclean;
use Moose::Util::TypeConstraints;

use XML::Feed;

has 'ua' => (
    is       => 'ro',
    required => 1,
    isa      => duck_type( [qw(get)] ),
);

has 'formatter' => (
    is       => 'ro',
    required => 1,
    isa      => duck_type( [qw(format_bytes)] ),
);

has 'renderer' => (
    is       => 'ro',
    required => 1,
    isa      => duck_type( [qw(process error)] ),
);

sub run {
    my ($self) = @_;
    my $html   = $self->fetch;
    my $data   = $self->find($html);
    $self->write($data);
}

sub fetch {
    my ($self) = @_;

    my $res = $self->ua->get('http://www.ezrss.it/feed');
    if ( $res->is_success ) {
        return $res->decoded_content;
    }
    else {
        die $res->status_line;
    }
}

sub find {
    my ( $self, $html ) = @_;
    my @data;
    my $feed = XML::Feed->parse( \$html );
    foreach my $item ( $feed->entries ) {
        next unless $item->title =~ /\s1x1\s/ || $item->title =~ /s01e01/i;
        next if $item->enclosure->length > 800_000_000;
        next if $item->title =~ /720p/i;
        push(
            @data,
            {
                title => $item->title,
                size =>
                  $self->formatter->format_bytes( $item->enclosure->length ),
                url => $item->enclosure->url,
            }
        );
    }
    return \@data if @data;
}

sub write {
    my ( $self, $data ) = @_;

    my $renderer = $self->renderer;
    my $out;
    $renderer->process( 'new_series.tt', { data => $data }, \$out )
      || die $renderer->error();
    return $out;
}

__PACKAGE__->meta->make_immutable;
1;
