package Winxle::BreadBoard;
use Moose;
use 5.012;
use Bread::Board;
use Class::Load;

sub setup {
    my ( $class, $env_class_name, @rest ) = @_;

    my $env_class = 'Winxle::BreadBoard::Env::' . $env_class_name;
    Class::Load::load_class($env_class);
    my $env = $env_class->new( name => $env_class_name );

    my $apps = container $class => ['Environment'] => as {
        container 'App' => as {
            service 'renderservice_worker' => (
                class        => 'Winxle::Service::Render::Worker',
                dependencies => {
                    yp            => '/Environment/yp_client',
                    coder         => '/Environment/json_coder',
                    data_provider => '/Environment/data_provider',
                    renderer      => '/Environment/TTnowrap',
                    localizer     => '/Environment/localizer',
                    root_dir => '/Component/Path/root',
                } );
            service 'yp_server' => (
                class => 'ZMQx::YellowPages::Server',
                dependencies =>
                    { address => '/Environment/yp_server_address', },
                parameters  => {
                    reset_service_store => { isa=>'Bool',default=>0 },
                }
            );
            service 'renderservice_ventilator' => (
                class        => 'Winxle::Service::Render::Ventilator',
                dependencies => { yp => '/Environment/yp_client' } );

            service 'web' => (
                class        => 'Winxle::Web',
                dependencies => {
                    yp              => '/Environment/yp_client',
                    renderer        => '/Environment/TT',
                    renderer_nowrap => '/Environment/TTnowrap',
                    dbic            => '/Component/DB/schema',
                    static_dir      => '/Component/Path/static',
                    session_store   => '/Component/Cache/session_store',
                    session_state   => '/Component/Cache/session_state_web',
                    localizer       => '/Environment/localizer',
                    json            => '/Environment/json_coder',
                    kioku           => '/Component/DataProvider/Kioku/dir',
                    config          => '/Component/Config/config',
                } );
            service 'admin' => (
                class        => 'Winxle::Admin',
                dependencies => {
                    renderer        => '/Environment/AdminTT',
                    renderer_nowrap => '/Environment/AdminTTnowrap',
                    dbic            => '/Component/DB/schema',
                    static_dir      => '/Component/Path/static',
                    session_store   => '/Component/Cache/session_store',
                    session_state   => '/Component/Cache/session_state_admin',
                    localizer       => '/Component/I18N/dummy',#'/Environment/localizer',
                    json            => '/Environment/json_coder',
                    kioku           => '/Component/DataProvider/Kioku/dir',
                    config          => '/Component/Config/config',
                    spreader_client => '/Component/ZMQ/spreader_client',
                } );

            service 'campaignspread' => (
                class        => 'Winxle::Service::CampaignSpread::Server',
                dependencies => {
                    yp    => '/Environment/yp_client',
                    dbh   => '/Component/DB/dbh',
                    renderer => '/Environment/TTnowrap',
                    root_dir => '/Component/Path/root',
                } );
            service 'l10n_service_backend' => (
                class        => 'I18N::GlueKit::LocalizerService',
                dependencies => {
                    yp    => '/Environment/yp_client',
                    dbh   => '/Component/DB/dbh',
                    cache => '/Component/Cache/CHI',
                } );
            service 'mailer' => (
                class=>'Winxle::Service::Mailer',
                dependencies => {
                    schema      => '/Component/DB/schema',
                    renderer    => '/Environment/TTnowrap',
                    localizer   => '/Environment/localizer',
                    config      => '/Component/Config/config',
                }
            );
        };

        # use include
        container 'Component' => as {
            container 'Path' => as {
                service 'home' => (
                    class     => 'Path::Class::Dir',
                    lifecycle => 'Singleton',
                    block     => sub {
                        ( my $pck = __PACKAGE__ . ".pm" ) =~ s{::}{/}g;
                        my $app_home = $INC{$pck};
                        $app_home =~ s{/lib/$pck$}{};
                        return Path::Class::Dir->new($app_home);
                    },
                );
                service 'root' => (
                    class     => 'Path::Class::Dir',
                    lifecycle => 'Singleton',
                    block     => sub {
                        my $s = shift;
                        return $s->param('home')->subdir(qw(root));
                    },
                    dependencies => ['home'],
                );
                service 'templates' => (
                    class     => 'Path::Class::Dir',
                    lifecycle => 'Singleton',
                    block     => sub {
                        my $s = shift;
                        return $s->param('home')->subdir(qw(root tt));
                    },
                    dependencies => ['home'],
                );
                service 'admin_templates' => (
                    class     => 'Path::Class::Dir',
                    lifecycle => 'Singleton',
                    block     => sub {
                        my $s = shift;
                        return $s->param('home')->subdir(qw(root admin_tt));
                    },
                    dependencies => ['home'],
                );
                service 'static' => (
                    class     => 'Path::Class::Dir',
                    lifecycle => 'Singleton',
                    block     => sub {
                        my $s = shift;
                        return $s->param('home')->subdir(qw(root static));
                    },
                    dependencies => ['home'],
                );
            };
            container 'JSON' => as {
                service 'pretty' => (
                    class     => 'JSON::XS',
                    lifecycle => 'Singleton',
                    block     => sub {
                        return JSON::XS->new->pretty->allow_nonref;
                    },
                );
                service 'terse' => (
                    class     => 'JSON::XS',
                    lifecycle => 'Singleton',
                    block     => sub {
                        return JSON::XS->new->allow_nonref;
                    },
                );
            };
            container 'Cache' => as {
                service 'CHI' => (
                    class        => 'CHI',
                    lifecycle    => 'Singleton',
                    dependencies => {
                        driver   => service( driver => "FastMmap" ),
                        root_dir => service(
                            root_dir => "/tmp/chi-driver-fastmmap-$<"
                        ),
                    } );
                service 'session_store' => (
                    class        => "Plack::Session::Store::Cache",
                    lifecycle    => 'Singleton',
                    dependencies => { cache => 'CHI', },
                );
                service 'session_state_web' => (
                    class        => "Plack::Session::State::Cookie",
                    lifecycle    => 'Singleton',
                    dependencies => { session_key=>service( session_key=>'winxle_web') },
                );
                service 'session_state_admin' => (
                    class        => "Plack::Session::State::Cookie",
                    lifecycle    => 'Singleton',
                    dependencies => { session_key=>service( session_key=>'winxle_admin') },
                );
            };
            container 'TT' => as {
                service 'default_params' => (
                    lifecycle => "Singleton",
                    dependencies =>
                        { include_path => '/Component/Path/templates', },
                    block => sub {
                        my $s = shift;
                        return {
                            TEMPLATE_EXTENSION => ".tt",
                            ERROR              => 'site/error.tt',
                            TIMER              => 0,
                            ENCODING           => 'utf-8',
                            CLASS              => "Template::AutoFilter",
                            PRE_PROCESS        => 'site/macros.tt',
                            INCLUDE_PATH => [ $s->param('include_path') ],
                        };
                    },
                );
                service 'wrapped' => (
                    lifecycle => "Singleton",
                    class     => 'Template::AutoFilter',
                    block     => sub {
                        my $s     = shift;
                        my %param = %{ $s->param('default_params') };
                        return Template::AutoFilter->new( %param,
                            WRAPPER => 'site/wrapper.tt', );
                    },
                    dependencies => ['default_params'],
                );
                service 'nowrap' => (
                    lifecycle => "Singleton",
                    class     => 'Template::AutoFilter',
                    block     => sub {
                        my $s     = shift;
                        my %param = %{ $s->param('default_params') };
                        return Template::AutoFilter->new( %param, );
                    },
                    dependencies => ['default_params'],
                );
            };
            container 'AdminTT' => as { # TODO too much copy from TT!
                service 'default_params' => (
                    lifecycle => "Singleton",
                    dependencies =>
                        { include_path => '/Component/Path/admin_templates', },
                    block => sub {
                        my $s = shift;
                        return {
                            TEMPLATE_EXTENSION => ".tt",
                            ERROR              => 'site/error.tt',
                            TIMER              => 0,
                            ENCODING           => 'utf-8',
                            CLASS              => "Template::AutoFilter",
                            PRE_PROCESS        => 'site/macros.tt',
                            INCLUDE_PATH => [ $s->param('include_path') ],
                        };
                    },
                );
                service 'wrapped' => (
                    lifecycle => "Singleton",
                    class     => 'Template::AutoFilter',
                    block     => sub {
                        my $s     = shift;
                        my %param = %{ $s->param('default_params') };
                        return Template::AutoFilter->new( %param,
                            WRAPPER => 'site/wrapper.tt', );
                    },
                    dependencies => ['default_params'],
                );
                service 'nowrap' => (
                    lifecycle => "Singleton",
                    class     => 'Template::AutoFilter',
                    block     => sub {
                        my $s     = shift;
                        my %param = %{ $s->param('default_params') };
                        return Template::AutoFilter->new( %param, );
                    },
                    dependencies => ['default_params'],
                );
            };
            container 'Renderer' => as {
                service 'TT' => (
                    class        => 'Winxle::Render::TT',
                    lifecycle    => "Singleton",
                    dependencies => {
                        tt        => '/Component/TT/wrapped',
                        localizer => '/Environment/localizer',
                    } );
                service 'TTnowrap' => (
                    class        => 'Winxle::Render::TT',
                    lifecycle    => "Singleton",
                    dependencies => {
                        tt        => '/Component/TT/nowrap',
                        localizer => '/Environment/localizer',
                    } );
                service 'AdminTT' => (
                    class        => 'Winxle::Render::TT',
                    lifecycle    => "Singleton",
                    dependencies => {
                        tt        => '/Component/AdminTT/wrapped',
                        localizer => '/Environment/localizer',
                    } );
                service 'AdminTTnowrap' => (
                    class        => 'Winxle::Render::TT',
                    lifecycle    => "Singleton",
                    dependencies => {
                        tt        => '/Component/AdminTT/nowrap',
                        localizer => '/Environment/localizer',
                    } );
            };
            container 'DB' => as {
                service 'db_connect' => (
                    lifecycle => "Singleton",
                    class     => "Winxle::DB::Connect",
                );
                service 'schema' => (
                    lifecycle => "Singleton",
                    block     => sub {
                        my $s = shift;
                        return $s->param('db_connect')->schema;
                    },
                    dependencies => ['db_connect'],
                );
                service 'dbh' => (
                    lifecycle => "Singleton",
                    block     => sub {
                        my $s = shift;
                        return $s->param('db_connect')->dbh;
                    },
                    dependencies => ['db_connect'],
                );
                service 'dsn' => (
                    lifecycle => "Singleton",
                    block     => sub {
                        my $s = shift;
                        return $s->param('db_connect')->dsn;
                    },
                    dependencies => ['db_connect'],
                );
            };
            container 'DataProvider' => as {
                container 'GoogleSpreadsheet' => as {
                    service 'username' => 'username@gmail.com';
                    service 'password' => '12345';
                    service 'document' =>
                        'Ohnae8edae3Ee7Othi6x';
                    service 'spreadsheet' => (
                        class => 'Winxle::DataProvider::GoogleSpreadsheet',
                        dependencies => {
                            username => depends_on('username'),
                            password => depends_on('password'),
                            document => depends_on('document'),
                        } );
                };
                container 'Kioku' => as {
                    service 'dir' => (
                        class => 'KiokuDB',
                        block => sub {
                            my $s = shift;
                            return KiokuDB->connect(
                                $s->param('dsn'),
                                typemap=>$s->param('typemap'),
                            );
                        },
                        dependencies => {
                            dsn=> '/Component/DB/dsn',
                            typemap => 'typemap'
                        },
                    );
                    service 'typemap' => (
                        class=>'KiokuDB::TypeMap',
                        block=>sub {
                            Class::Load::load_class('KiokuDB::TypeMap::Entry::Naive');
                            my $i = KiokuDB::TypeMap::Entry::Naive->new(
                                intrinsic=>1
                            );

                            return KiokuDB::TypeMap->new(
                                isa_entries => {
                                    'Winxle::DataWrapper::Intrinsic' => $i,
                                },
                            );
                        },
                    );
                };
            };
            container 'I18N' => as {
                service 'backend_zmq' => (
                    class        => 'I18N::GlueKit::Localizer::BackendZMQ',
                    dependencies => { yp => '/Environment/yp_client', } );
                service 'localizer_zmq' => (
                    class        => 'I18N::GlueKit::Localizer',
                    dependencies => {
                        localizer_backend => 'backend_zmq',
                        cache             => '/Component/Cache/CHI',
                    } );
                service 'data_localize' => (
                    class        => "I18N::GlueKit::Localize",
                    service_type => "Setter",
                    dependencies =>
                        { add_localizer => depends_on('localizer_zmq'), },
                );
                service 'dummy' => (
                    class=>'Winxle::DummyLocalizer',
                    lifecycle => "Singleton",
                );
            };
            container 'ZMQ' => as {
                service 'spreader_client' => (
                    class=>'Winxle::Service::CampaignSpread::Client',
                    dependencies=> {
                        yp              => '/Environment/yp_client',
                    },
                );
            };
            container 'Config' => as {
                service 'config' => (
                    lifecycle => "Singleton",
                    class=>'Config::JFDI',
                    dependencies => {
                        name=>service( name => "winxle" ),
                        path=>'/Component/Path/home',
                    },
                );
            }
        };
    };

    my $bb = $apps->create( 'Environment' => $env, @rest );
    return $bb;
}

1;
