package testlib::OxController;
use strict;
use warnings;
use Moose;
use Winxle::BreadBoard;
use namespace::autoclean;
use OX::Request;
use Plack::Response;
use Test::Most;
use Carp qw(croak);
use Data::Dumper;

has 'bb' => (
    is=>'ro',
    default => sub {
        Winxle::BreadBoard->setup('Test');
    }
);

sub init {
    my ($self, $name, $args) = @_;
    $args ||= {};

    my $controller_class = $self->controller_base."::Controller::".$name;
    Class::Load::load_class($controller_class);

    my $bb = $self->bb;
    my %args = (
        %$args,
        # TODO figure out what a given controllor needs
        # should be possible somehow, as OX isa BB
        renderer        => $bb->resolve(service=>'/Environment/TT'),
        renderer_nowrap => $bb->resolve(service=>'/Environment/TTnowrap'),
        json            => $bb->resolve(service=>'/Environment/json_coder'),
        localizer       => $bb->resolve(service=>'/Environment/localizer'),
        config          => $bb->resolve(service=>'/Component/Config/config'),
        dbic            => $bb->resolve(service=>'/Component/DB/schema'),
    );

    my $controller = $controller_class->new( %args );
    return $controller;
}

sub init_dbic_txn {
    my ($self, $name, $args) = @_;
    my $c = $self->init($name, $args);
    my $dbic = $c->dbic;
    $dbic->txn_begin;
    return ($c,$dbic);
}

sub request {
    my ($self, $args) = @_;
    $args||={};

    my $env = $args->{env} || {};
    if ($args->{params}) { # TODO this is a bit too simplistic
        $env->{QUERY_STRING} = join('&',map { $_.'='.$args->{params}->{$_} } keys %{$args->{params}})
    }
    $env->{'psgix.session'} ||= $args->{session} || {};

    # TODO add/fake more common request data?

    return OX::Request->new_from_env($env);
}

sub call {
    my ($self, $req, $controller, $action, $args) = @_;
    $args||=[];
    my $rv;
    lives_ok {
        $rv = $controller->$action($req, @$args);
    } "called $action on ".ref($controller);
    return $rv
}

sub call_plack_res {
    my $self = shift;
    my $raw = $self->call( @_ );
    if ($raw->[0] =~ /^\d+$/) {
        return Plack::Response->new(@$raw);
    }
    else {
        croak("Return value does not look like PSGI:\n".Dumper($raw));
    }
}

__PACKAGE__->meta->make_immutable;
1;
