use 5.010;
use strict;
use warnings;
use lib 't/';
use utf8;

use Winxle::Run;
use Test::Most;

use testlib::Web;
use testlib::Fixtures::User;

my $t = testlib::Web->new;
my ($controller, $dbic) = $t->init_dbic_txn('User');
my $user = testlib::Fixtures::User->new(dbic=>$dbic)->bart;

subtest 'successful_login' => sub {
    my $req = $t->request({
        params=> {
            email=>$user->email,
            password=>'12345',
        }
    });
    my $res = $t->call_plack_res($req, $controller, 'login');

    is($req->session->{user_id},$user->id,'user id in session');
    is($res->status,302,'status: redirect');
    is($res->location,'/','redirect to /');
};

subtest 'login_form' => sub {
    my $req = $t->request;
    my $res = $t->call($req, $controller, 'login');

    is($req->session->{user_id},undef,'no user in session');
    is($res->[0],'user/login.tt','correct template');
};

subtest 'no_password' => sub {
    my $req = $t->request({
        params=> {
            email=>$user->email,
        }
    });
    my $res = $t->call($req, $controller, 'login');

    is($req->session->{user_id},undef,'user id in session');
    my $form = $res->[1]->{form};
    is($form->num_errors,1,'one error in form');
    ok($form->field('password')->errors->[0],'password missing');
};

subtest 'bad credentials' => sub {
    my $req = $t->request({
        params=> {
            email=>$user->email,
            password=>'hase',
        }
    });
    my $res = $t->call($req, $controller, 'login');

    is($req->session->{user_id},undef,'user id in session');
    # TODO handle errors
};

done_testing();
$ENV{NO_CLEANUP} ? $dbic->txn_commit : $dbic->txn_rollback;

